##############################################################
# This code is for the study of:
#   Throwing light on dark diversity of vascular plants in Chian
#    author: Lili Tang
#    reference1: P??rtel, M., Szava-Kovats, R., & Zobel, M. (2011). Dark diversity: shedding light on absent species. Trends in Ecology & Evolution, 26, 124-128.
#    reference2: Jari Oksanen, F. Guillaume Blanchet, Michael Friendly, Roeland Kindt, Pierre Legendre, Dan
#                McGlinn, Peter R. Minchin, R. B. O'Hara, Gavin L. Simpson, Peter Solymos, M. Henry H. Stevens,
#                Eduard Szoecs and Helene Wagner (2018). vegan: Community Ecology Package. R package version
#                2.5-3. https://CRAN.R-project.org/package=vegan
#    reference3: A. Liaw and M. Wiener (2002). Classification and Regression by randomForest. R News 2(3), 18--22.    
#
##############################################################
#
# 2.3 Estimation of dark diversity and community conpletenes
#
#  a. create region X species matrix 
#
#   Data source : http://frps.eflora.cn/
# 
#   The matrix row names and their refered region names:
#     10+12+13: Beijing + Tianjin + Hebei 
#     15: Neimenggu
#     22: Jilin 
#     23: Heilongjiang
#     21: Liaoning 
#     62+64: Gansu + Ningxia 
#     61: Shaanxi 
#     14: Shanxi 
#     37: Shandong 
#     41: Henan 
#     31+32:  Shanghai + Jiangsu
#     34: Anhui 
#     33: Zhejiang 
#     42: Hubei 
#     50+51: Chongqing + Sichuan 
#     65: Xinjiang 
#     63: Qinghai 
#     54: Tibet 
#     53: Yunnan 
#     52: Guizhou 
#     43: Hunan 
#     36: Jiangxi 
#     35: Fujian 
#     71: Taiwan 
#     45: Guangxi 
#     44+81+91: Guangdong + Hongkong + Macau 
#     46: Hainan 
#
codeList = c("10+12+13","15","22","23","21","62+64","61","14","37","41","31+32","34","33","42","50+51","65","63","54", "53","52","43","36","35","71","45","44+81+91","46")

# start to generate region X species matrix
bufferTable = data.frame()
for(i in 1:length(codeList)) {
  fileName = paste(codeList[i],".csv",sep='')
  csvFile = read.csv(fileName,header=FALSE)
  nameList = unique(as.vector(csvFile[,1]))
  # creat a vector newRow(referring to new area), all values are 0, size =ncol(bufferTable)
  newRow = integer(ncol(bufferTable)) 
  # add the newRow into bufferTable
  bufferTable = rbind(bufferTable,newRow) 
  
  # following loop is for checking if there is any species in newRow(new area) that already existed in bufferTable
  # if there is, change its value from 0 to 1, and remove this species from newRow
  for(j in 0:ncol(bufferTable)) {
    if (j == 0) {
      next
    }
    if(colnames(bufferTable)[j] %in% nameList) {
      bufferTable[i,colnames(bufferTable)[j]] = 1
      nameList = nameList[! nameList %in% colnames(bufferTable)[j]]
    }
  }
  
  # for the rest species in newRow, all of them have not appeared in bufferTabe
  # so we create a new column in bufferTable, mark all previous areas in this column 0, and the new area in this column 1
  for(j in 0:length(nameList)) {
    if (j == 0) {
      next
    }
    newVector = integer(i)
    newVector[i] = 1
    if (ncol(bufferTable) == 0) {
      bufferTable = as.data.frame(newVector)
    } else {
      bufferTable = cbind(bufferTable,newVector)
    }
    # name of the new column = species name
    colnames(bufferTable)[ncol(bufferTable)] = nameList[j]
    print(paste(nameList[j]," added in. Species in total = ", ncol(bufferTable), sep= " "))
  }
  # name of the new row = area code
  row.names(bufferTable)[i] = codeList[i]
  print(paste(codeList[i]," done. Row number: ",i,sep=" "))
}



#   b. Estimating dark diversity
#
library(vegan)
table=bufferTable

## preparing an empty table for dark diversity, dimensions as obs table
dark1=table
dark1[,]=0

## Number of species and plots in the table
nospp=ncol(table)
noplot=nrow(table)

## Co-occurrence probability for each region and species
Tprobability=beals(table,include=FALSE)

## For each species we need co-occurrence threshold:
## We calculate 99% quantile (1th percentile) for these regions where specific species is present
thresh=numeric(nospp)
for (j in 1:nospp) {thresh[j]=quantile(Tprobability[table[,j]>0,j],probs=0.01,na.rm = T)}

## we determine dark diversity: species which are absent but often co-occur with present spp
for (i in 1:noplot) {dark1[i,Tprobability[i,]>thresh & table[i,]==0]=1}

dark2 = dark1

# darkClean keeps all dark species and eliminates the species that are not "dark" for all regions
darkClean= dark2[, colSums(dark2 != 0) > 0] 

write.csv(darkClean, "./dark_diversity.csv") 


################################################################################################
#
# 2.4 Assessing variables that affect dark diversity
library(randomForest)
#   read the dark diversity X bioclimatic variables matrix
BIO <- read.csv("WC2.0_BIO1_TO_BIO19.csv",header = T)
#   Normalize the values in the matrix
BIOScal <- scale(BIO,center =T, scale = T)
#   run the random forest model
RF <- randomForest(dark_diver ~ ., data=BIOScal,  ntree=100,mtry=3,
                        importance=TRUE)
#   extract the importance of every bioclimatic variables
RFIM <-importance(RF)

write.csv(RFIM,file="RFIM.csv")
################################################################################################
# 2.5 Predicting the distributon of rare dark species with future climate change
#
library("raster")
# read the current bioclimatic variables
C_2 <- raster("BIO2.asc")
C_3 <- raster("BIO3.asc")
C_4 <- raster("BIO4.asc")
C_5 <- raster("BIO5.asc")
C_7 <- raster("BIO7.asc")
C_8 <- raster("BIO8.asc")
C_9 <- raster("BIO9.asc")
C_10 <- raster("BIO10.asc")
C_11 <- raster("BIO11.asc")
C_14 <- raster("BIO14.asc")
C_15 <- raster("BIO15.asc")
C_17 <- raster("BIO17.asc")
C_19 <- raster("BIO19.asc")

# stacking the current bioclimatic variables to process them at one go 
clim_C <- raster::stack(C_2,C_3,C_4,C_5,C_7,C_8,C_9,C_10,C_11,C_14,C_15,C_17,C_19) 
# to run correlations between different layers
pairs(clim_C[[1:13]])


# read the 2050s bioclimatic variables
F_50_2 <- raster("BIO2.asc")
F_50_3 <- raster("BIO3.asc")
F_50_4 <- raster("BIO4.asc")
F_50_5 <- raster("BIO5.asc")
F_50_7 <- raster("BIO7.asc")
F_50_8 <- raster("BIO8.asc")
F_50_9 <- raster("BIO9.asc")
F_50_10 <- raster("BIO10.asc")
F_50_11 <- raster("BIO11.asc")
F_50_14 <- raster("BIO14.asc")
F_50_15 <- raster("BIO15.asc")
F_50_17 <- raster("BIO17.asc")
F_50_19 <- raster("BIO19.asc")

# stacking the 2050s bioclimatic variables to process them at one go 
clim_F_50 <- raster::stack(F_50_2, F_50_3,F_50_4,F_50_5,F_50_7,F_50_8,F_50_9,F_50_10,F_50_11,F_50_14,F_50_15,F_50_17,F_50_19) 

# to run correlations between different layers
pairs(clim_F_50[[1:13]])


# read the 2070s bioclimatic variables
F_70_2 <- raster("BIO2.asc")
F_70_3 <- raster("BIO3.asc")
F_70_4 <- raster("BIO4.asc")
F_70_5 <- raster("BIO5.asc")
F_70_7 <- raster("BIO7.asc")
F_70_8 <- raster("BIO8.asc")
F_70_9 <- raster("BIO9.asc")
F_70_10 <- raster("BIO10.asc")
F_70_11 <- raster("BIO11.asc")
F_70_14 <- raster("BIO14.asc")
F_70_15 <- raster("BIO15.asc")
F_70_17 <- raster("BIO17.asc")
F_70_19 <- raster("BIO19.asc")

# stacking the 2070s bioclimatic variables to process them at one go
clim_F_70 <- raster::stack(F_70_2, F_70_3,F_70_4,F_70_5,F_70_7,F_70_8,F_70_9,F_70_10,F_70_11,F_70_14,F_70_15,F_70_17,F_70_19)

# to run correlations between different layers
pairs(clim_F_70[[1:13]])


## use the left environmental variables for current, 2050s and 2070s
clim_C <- raster::stack(C_2,C_3,C_5,C_7,C_11,C_14,C_15) 
pairs(clim_C[[1:7]])

clim_F_50 <- raster::stack(F_50_2, F_50_3,F_50_5,F_50_9,F_50_15,F_50_19) 
pairs(clim_F_50[[1:6]])

clim_F_70 <- raster::stack(F_70_3,F_70_5,F_70_7,F_70_11,F_70_17,F_70_19)
pairs(clim_F_70[[1:6]])

################################################################################################
################################################################################################


